package com.cesgroup.zw.core.codetable.xml;


import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import com.cesgroup.zw.core.codetable.Code;
import com.cesgroup.zw.core.codetable.util.StringUtils;

public class CodeTableXml {

	protected CodeTableXmlHandler handler;


	protected XmlNode root;


	protected ClassLoader classLoader;


	protected int retryCount;


	protected long lastModified;


	private File xmlFile;


	private InputStream xmlStream;


	private boolean isDynamicLoad;


	public CodeTableXml(String filepath) {
		this(filepath, 3, false);
	}
	
	public CodeTableXml(InputStream xmlStream) {
		this(xmlStream, 3, false);
	}
	

	public CodeTableXml(String filepath, boolean isOgnl) {
		this(filepath, 3, isOgnl);
	}

	public CodeTableXml(String filepath, int retryCount, boolean isOgnl) {
		if (this.xmlFile == null) {
			this.xmlStream = this.getClass().getClassLoader().getResourceAsStream(filepath);
		}
		this.retryCount = retryCount;
		this.handler = new CodeTableXmlHandler(isOgnl);
		this.isDynamicLoad = false;
		this.load();
	}

	public CodeTableXml(InputStream xmlStream, int retryCount, boolean isOgnl) {
		if (this.xmlStream == null) {
			this.xmlStream = xmlStream;
		}
		this.retryCount = retryCount;
		this.handler = new CodeTableXmlHandler(isOgnl);
		this.isDynamicLoad = false;
		this.load();
	}
	
	public XmlNode getRootNode() {
		this.update(0);
		return this.root;
	}

	public void setDynamicLoad(boolean isDynamicLoad) {
		this.isDynamicLoad = isDynamicLoad;
	}

	public <T extends Object> List<T> copyToBean(String xpath, Class<T> clazz) {
		return this.copyToBean(xpath, clazz, null);
	}

	public <T extends Object> List<T> copyToBean(String xpath, Class<T> clazz, String attName) {
		List<T> beanList = new ArrayList<T>();
		try {
			List<XmlNode> nodeList =
				this.getRootNode().getNodeListByXPath(xpath);
			Field[] fields = clazz.getDeclaredFields();
			for (XmlNode node : nodeList) {
	
				List<XmlNode> children = node.getChildren();
				T bean = clazz.newInstance();
				for (Field field : fields) {
					field.setAccessible(true);
					List<XmlNode> selectedChildren =
						node.getChildren(field.getName());
					if (selectedChildren != null && selectedChildren.size() > 0) {
						field.set( bean, selectedChildren.get(0).getValue());
					} else if (node.getAttributes().containsKey(field.getName())) {
						field.set(bean, node.getAttributes().get(field.getName()));
					} else if (StringUtils.isNotEmpty(attName)) {
						for (XmlNode child : children) {
							if (StringUtils.equals(
									child.getAttributes().get(attName),
									field.getName())) {
								field.set(bean, child.getValue());
							}
						}
					}
				}
				beanList.add(bean);
			}
		} catch (Exception e) {
			
		}
		return beanList;
	}


	public <T extends Object> Map<String, T> copyToBeanMap(String xpath, Class<T> clazz, String keyField) {
		return this.copyToBeanMap(xpath, clazz, null, keyField);
	}

	public <T extends Object> Map<String, T> copyToBeanMap(String xpath, Class<T> clazz, String attName, String keyField) {
		List<T> beanList = this.copyToBean(xpath, clazz, attName);
		return beanListToMap(beanList, clazz, keyField);
	}

	public List<XMLGroup> getXMLGroupList() throws Exception {
		
		List<XMLGroup> xmlList = new ArrayList<XMLGroup>();
		
		if(null != this.getRootNode()) {
			
			List<XmlNode> list = this.getRootNode().getChildren();
			
			if(null != list && !list.isEmpty()) {
				
				for(XmlNode node:list) {
					XMLGroup xmlGroup = new XMLGroup();
					
					xmlGroup.setPrimaryField(node.getAttributes().get("primaryField"));
					xmlGroup.setGroupName(node.getAttributes().get("name"));
					xmlGroup.setXpath("group@name("+xmlGroup.getGroupName()+")/code");
					String xmlGroupClazzName = Code.class.getName();
					
					if(null != node.getChildren("code-table-class") && !node.getChildren("code-table-class").isEmpty()) {
						xmlGroupClazzName = (String) node.getChildren("code-table-class").get(0).getValue();
					}
					if(null == xmlGroupClazzName) {
						xmlGroupClazzName = Code.class.getName();
					}
					xmlGroup.setClazz(Class.forName(xmlGroupClazzName));  
					xmlList.add(xmlGroup);
				}
				
			}
			
		} 
		
		return xmlList;
	}	
	
	static <T extends Object> Map<String, T>  beanListToMap(List<T> beanList,Class<T> clazz,String keyField){
		try {
			Map<String, T> beanMap = new LinkedHashMap<String, T>();

			Field field = clazz.getDeclaredField(keyField);
			field.setAccessible(true);
			for (T bean : beanList) {
				beanMap.put((String)field.get(bean), bean);
			}
			return beanMap;
		} catch (Exception e) {
			
		}
		return null;
	}

	public boolean isUpdate() {
		if (this.xmlFile == null) {
			return false;
		}
		return this.isDynamicLoad &&
			this.lastModified < this.xmlFile.lastModified();
	}


	protected void load() {
		synchronized (this) {
			try {
				SAXParserFactory spfactory = SAXParserFactory.newInstance();
				SAXParser parser = spfactory.newSAXParser();
				this.handler.initialize();
				if (this.xmlFile != null &&
						this.xmlFile.exists() && this.xmlFile.isFile()) {
					parser.parse(this.xmlFile, this.handler);
					this.root = this.handler.getRootNode();
				} else if (this.xmlStream != null) {
					parser.parse(this.xmlStream, this.handler);
					this.root = this.handler.getRootNode();
				}
			} catch (Exception e) {
				
			}
		}
	}


	private void update(int retry) {
		if (this.isUpdate()) {
			try {
				this.load();
			} catch (Exception e) {
				if (retry < this.retryCount) {
					this.update(retry+1);
					return;
				}
			}
			this.lastModified = (this.xmlFile == null) ?
					0L : this.xmlFile.lastModified();
		}
	}
}
